local concatTable

local new, del
do
	local cache = setmetatable({}, { __mode = 'k' })
	function new()
		local t = next(cache)
		if t then
			cache[t] = nil
		else
			t = {}
		end
		return t
	end
	function del(t, r)
		for k in next, t do
			if type(t[k]) == 'table' then
				del(t[k])
			end
			t[k] = nil
		end
		if r then
			return t
		end
		cache[t] = true
	end
end

--[[---------------------------------------------------------------------------------
 MOUNT-RELATED FUNCTIONS
------------------------------------------------------------------------------------]]
local PT = AceLibrary('PeriodicTable-3.0')
SacredBuff.mountCache = setmetatable({}, {
	__mode = 'v',
	__index = function(t, k)
		local mType, mQuality = select(3, k:find("([%a%']+)(%d+)"))
		
		assert(mType and mQuality, ('Invalid key request. Got %s.'):format(tostring(k)))
		
		local setString = PT:GetSetStringUncompressed('Misc.Mount.'..mType)
		if not concatTable then
			concatTable = {}
		end
		
		for itemId, speed in setString:gmatch('(%d+):(%d+)') do
			if speed == mQuality then
				tinsert(concatTable, itemId)
			end
		end
		
		local v = table.concat(concatTable, ',')
		
		concatTable = del(concatTable, true)
		
		if not v:find('(%d)') then
			v = nil
		end
		
		t[k] = v
		
		return t[k]
	end
})

function SacredBuff:IterateMounts(mountType)
	local setString = self.mountCache[mountType]
	
	if not self.mounts then
		self.mounts = {}
	end
	
	if not concatTable then
		concatTable = {}
	end
	
	for itemId in setString:gmatch('(%d+)') do
		if GetItemCount(tonumber(itemId)) > 0 then
			tinsert(concatTable, itemId)
		end
	end
	
	self.mounts[mountType] = table.concat(concatTable, ',') or nil
	
	concatTable = del(concatTable, true)
end

function SacredBuff:UpdateMountLocs()
	if self.db.char.playerLevel < 40 or self.db.char.ridingSkillIndex == 0 then return end
	
	local ridingSkill = select(4, GetSkillLineInfo(self.db.char.ridingSkillIndex))
	if ridingSkill < 75 then return end
	
	self:IterateMounts('Normal60')
	
	if ridingSkill >= 150 then
		self:IterateMounts('Normal100')
		self:IterateMounts("Ahn'Qiraj100")
		
		if ridingSkill >= 225 then
			self:IterateMounts('Flying60')
			
			if ridingSkill == 300 then
				self:IterateMounts('Flying280')
			end
		end
	end
	
	self:UpdateMountMenu()
end

function SacredBuff:InOutlands()
	if not self.outlandZones then
		self.outlandZones = strjoin(',', 'Twisting Nether', GetMapZones(3))
	end
	if self.outlandZones:find(GetRealZoneText()) then
		return true
	end
end

function SacredBuff:NewMountOption(k, zone)
	local mountName = GetItemInfo(tonumber(k))
	return {
		name = mountName, type = 'toggle',
		desc = mountName, isRadio = true,
		get = function()
			if self.db.char[zone..'Mount'] and self.db.char[zone..'Mount'] == mountName then
				return true
			end
		end,
		set = function()
			if self.frame.mount and not InCombatLockdown() then
				local inOutlands = self:InOutlands()
				if zone == 'outlands' and inOutlands or zone == 'azeroth' and not inOutlands then
					self.frame.mount:SetAttribute('macrotext', '/cast '..mountName)
					
					self.frame.mount.mountName = mountName
				end
				self.db.char[zone..'Mount'] = mountName
			end
		end
	}
end

function SacredBuff:UpdateMountMenu()
	
	local zoneTable = self.consoleArgs.args.Mount.args['Azeroth'].args
	for k in next, zoneTable do
		if GetItemCount(tonumber(k)) == 0 then
			k = del(k)
		end
	end
	
	if not concatTable then
		concatTable = {}
	end
	
	for k, v in next, self.mounts do
		if k:find('^Normal') then
			tinsert(concatTable, v)
		end
	end
	
	local zoneMounts = table.concat(concatTable, ',')
	
	concatTable = del(concatTable, true)
	
	for mount in zoneMounts:gmatch('(%d+)') do
		if not zoneTable[mount] then
			zoneTable[mount] = self:NewMountOption(mount, 'azeroth')
		end
	end
	
	zoneTable = self.consoleArgs.args.Mount.args['Outlands'].args
	for k in next, zoneTable do
		if GetItemCount(tonumber(k)) == 0 then
			k = del(k)
		end
	end
	
	for k, v in next, self.mounts do
		if k:find('^Flying') then
			tinsert(concatTable, v)
		end
	end
	
	local zoneMounts = table.concat(concatTable, ',')
	
	concatTable = del(concatTable, true)
	
	for mount in zoneMounts:gmatch('(%d+)') do
		if not zoneTable[mount] then
			zoneTable[mount] = self:NewMountOption(mount, 'outlands')
		end
	end
	

end


