﻿--[[---------------------------------------------------------------------------------
 ADDON DECLARATION && SAVED VARIABLES && MENU INITIALIZATION
------------------------------------------------------------------------------------]]
SacredBuff = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")

local L = AceLibrary("AceLocale-2.2"):new("SacredBuff")
local BS = AceLibrary("Babble-Spell-2.2")
local M = AceLibrary("Metrognome-2.0")
local G = AceLibrary("Gratuity-2.0")
local _G = getfenv(0)

local function DelTable(t, r)
	if not t then
		return
	end
	for k in next, t do
		if type(t[k]) == 'table' then
			t[k] = DelTable(t[k])
		else
			t[k] = nil
		end
	end
	if r then
		return t
	end
end

-- BINDINGS
-----------------------------------------------------------
BINDING_HEADER_SACREDBUFF = L["SacredBuff"]
BINDING_NAME_MOUNT = L["Mount"]

-- SAVED VARIABLES
-----------------------------------------------------------
SacredBuff:RegisterDB("SBDB", "SBPCDB")

SacredBuff:RegisterDefaults("profile", {
	movable = true,
	tooltip = 2,
	toggle = true,
	scaleSize = 1
})

SacredBuff:RegisterDefaults("char", {
	groupMode = 1, -- 1 == false; 2 == true
})

SacredBuff.spells = {
	[ BS["Inner Fire"] ] = false,
	[ BS["Resurrection"] ] = false, 
	
	[ BS["Divine Spirit"] ] = false,
	[ BS["Power Word: Fortitude"] ] = false,
	[ BS["Shadow Protection"] ] = false,
	
	["TALENT"] = false,
	["RACIAL"] = false,
	
	[ BS["Prayer of Spirit"] ] = false,
	[ BS["Prayer of Fortitude"] ] = false,
	[ BS["Prayer of Shadow Protection"] ] = false
}

SacredBuff.talents = {
	BS["Pain Suppression"],
	BS["Circle of Healing"],
	BS["Vampiric Touch"],
	BS["Power Infusion"],
	BS["Lightwell"],
	BS["Shadowform"],
	BS["Inner Focus"]
}

SacredBuff.racials = {
	BS["Touch of Weakness"],
	BS["Shadowguard"],
	BS["Fear Ward"],
	BS["Elune's Grace"],
	BS["Feedback"]
}

-- MENU INITIALIZATION
-----------------------------------------------------------
SacredBuff.consoleArgs = {
	type = "group",
	args = {
		
		[ L["Lock"] ] = {
			name = L["Lock"], type = "toggle",
			desc = L["(Un)Lock the SacredBuff frame."],
			map = { [false] = "|cffff0000"..L["On"].."|r", [true] = "|cff00ff00"..L["Off"].."|r" },
			get = function()
				return SacredBuff.db.profile.movable
			end,
			set = function()
				SacredBuff.db.profile.movable = not SacredBuff.db.profile.movable
				if SacredBuff.db.profile.movable then
					SacredBuff.frame.title:Show()
					SacredBuff.frame:SetBackdropColor(0.00, 0.00, 0.00, 0.60)
				else
					SacredBuff.frame.title:Hide()
					SacredBuff.frame:SetBackdropColor(0.00, 0.00, 0.00, 0.00)
				end
			end
		},
		[ L["Scale"] ] = {
			name = L["Scale"], type = "range",
			desc = L["Set the frame's scale."],
			min = 0.50, max = 2.00, isPercent = true,
			get = function() return SacredBuff.db.profile.scaleSize end,
			set = function(v)
				SacredBuff.db.profile.scaleSize = v
				SacredBuff:ScaleFrame()
			end,
		},
		['Mount'] = {
			name = 'Mount Options', type = 'group',
			desc = 'A myriad of Mount options.',
			hidden = function()
				return SacredBuff.db.char.playerLevel < 40
			end,
			args = {
				['Azeroth'] = {
					name = 'Azeroth Mount', type = 'group',
					desc = 'Select your preferred mount for your travels in Azeroth.',
					hidden = function()
						return not SacredBuff.mounts['Normal60'] and not SacredBuff.mounts['Normal100']
					end,
					args = {}
				},
				['Outlands'] = {
					name = 'Outlands Mount', type = 'group',
					desc = 'Select your preferred mount for your adventures in the Outlands.',
					hidden = function()
						return not SacredBuff.mounts['Normal60'] and not SacredBuff.mounts['Normal100'] and
						   not SacredBuff.mounts['Flying60'] and not SacredBuff.mounts['Flying280']
					end,
					args = {}
				}
			}
		},
		[ L["Tooltip"] ] = {
			name = L["Tooltip Options"], type = "group",
			desc = L["Select what information the tooltip provides."],
			args = {
				[ L["None"] ] = {
					name = L["No Tooltip"], type = "execute",
					desc = L["No tooltip will be displayed over the spell icons."],
					func = function()
						if SacredBuff.db.profile.tooltip ~= 0 then
							SacredBuff.db.profile.tooltip = 0
							SacredBuff:Print(L["Tooltips turned off."])
						end
					end
				},
				[ MANA ] = {
					name = L["Mana Only"], type = "execute",
					desc = L["Only the mana cost of the spell will be displayed in its tooltip."],
					func = function()
						if SacredBuff.db.profile.tooltip ~= 1 then
							SacredBuff.db.profile.tooltip = 1
							SacredBuff:Print(L["Tooltips will now display the spell's mana cost."])
						end
					end
				},
				[ L["Full"] ] = {
					name = L["Full Tooltip"], type = "execute",
					desc = L["The spell's full tooltip will be displayed on mouseover."],
					func = function()
						if SacredBuff.db.profile.tooltip ~= 2 then
							SacredBuff.db.profile.tooltip = 2
							SacredBuff:Print(L["Full spell tooltips will now be displayed."])
						end
					end
				}
			}
		}
	}
}
SacredBuff:RegisterChatCommand(L["Chat_Commands"], SacredBuff.consoleArgs)

--[[---------------------------------------------------------------------------------
 ADDON INITIALIZATION && ACTIVATION
------------------------------------------------------------------------------------]]
function SacredBuff:OnInitialize()
	if select(2, UnitClass("player")) ~= "PRIEST" then
		self:Print(L["You are not a Priest."])
		self:ToggleActive(false)
		self:Print(L["This addon has been disabled."])
		return
	else
		self:ToggleActive(true)
	end
	
	self.revision = tonumber(('$Revision: 30706 $'):sub(-7, -3))
	self.version = self.version .. ' |cffff8888r' .. self.revision .. '|r'
	self.date = ('$Date: 2007-03-21 23:06:37 -0400 (Wed, 21 Mar 2007) $'):sub(8, 17)
	
	if not self.db.char.revision then
		local charDB = self.db.char
		if charDB.Mounts60 then
			charDB.Mounts60 = DelTable(charDB.Mounts60)
		end
		if charDB.Mounts100 then
			charDB.Mounts100 = DelTable(charDB.Mounts100)
		end
		if charDB['Mounts - AQ100'] then
			charDB['Mounts - AQ100'] = DelTable(charDB['Mounts - AQ100'])
		end
		if charDB.prefMount then
			charDB.prefMount = DelTable(charDB.prefMount)
		end
		if charDB.prefAQMount then
			charDB.prefAQMount = DelTable(charDB.prefAQMount)
		end
		if charDB.sacredCount then
			charDB.sacredCount = nil
		end
	end
	
	self:CreateTooltip()
	self:CreateMainFrame()
	
--	M:Register("SacredBuff_ButtonsUsable", self.UpdateButtonsUsable, 0.25, self)
end

function SacredBuff:OnEnable()
	self:Print("THIS IS CURRENTLY UNDER HEAVY DEVELOPEMENT! Revert if you want something that works. That is all.")
	if not self.db.char.playerLevel then
		self.db.char.playerLevel = UnitLevel("player")
	end
	
	if not self.db.char.ridingSkillIndex then
		self:SKILL_LINES_CHANGED()
	end
	
	self:RegisterBucketEvent("BAG_UPDATE", 1)
	
	self:RegisterEvent("ZONE_CHANGED", "BAG_UPDATE")
	self:RegisterEvent("LEARNED_SPELL_IN_TAB")
	self:RegisterEvent("PLAYER_LEVEL_UP")
	self:RegisterEvent("SKILL_LINES_CHANGED")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	if InCombatLockdown() then
	--	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	else
	--	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	end
	
	self:LEARNED_SPELL_IN_TAB()
	
	for _, s in ipairs(self.racials) do
		local r = self:FindSpell(s)
		if r then
			self.spells["RACIAL"] = r
			break
		end
	end
	
	self:UpdateCandleCount()
	
	self.frame:Show()
	
	self:ScheduleEvent(self.UpdateButtons, 0.5, self)
end

function SacredBuff:OnDisable()
	self.frame:Hide()
end


--[[---------------------------------------------------------------------------------
 EVENT HANDLERS
------------------------------------------------------------------------------------]]
function SacredBuff:BAG_UPDATE()
	self:UpdateCandleCount()
	self:UpdateMountLocs()
end

function SacredBuff:LEARNED_SPELL_IN_TAB()
	local tbl
	for s in next, self.spells do
		tbl = self:FindSpell(s)
		self.spells[s] = tbl or false
	end
	tbl = DelTable(tbl)
	for i, v in ipairs(self.talents) do
		tbl = self:FindSpell(v)
		if tbl then
			self.spells["TALENT"] = tbl
			return
		end
	end
end

function SacredBuff:PLAYER_LEVEL_UP(newLevel)
	self.db.char.playerLevel = newLevel
end

function SacredBuff:SKILL_LINES_CHANGED()
	for i = 1, GetNumSkillLines(), 1 do
		local skillName = GetSkillLineInfo(i)
		if skillName == L["Riding"] then
			local oldSkillIndex = self.db.char.ridingSkillIndex
			self.db.char.ridingSkillIndex = i
			if oldSkillIndex == 0 then
				self:UpdateMountLocs()
			end
			return
		end
	end
	self.db.char.ridingSkillIndex = 0
end

function SacredBuff:PLAYER_TARGET_CHANGED()
	local mode = self.db.char.groupMode
	local spell, inRange, hasTarget
	if UnitExists("target") then
		hasTarget = true
	end
	for k, v in next, self.frame do
		if self.spells[k] and v.states then
			spell = v.states[mode] or v.states[1]
			if hasTarget then
				inRange = IsSpellInRange(spell, "target")
			end
			if inRange == 0 and not v.outOfMana then
				v.icon:SetVertexColor(0.80, 0.10, 0.10)
				v.outOfRange = true
			elseif v.outOfRange then
				if v.outOfMana then
					v.icon:SetVertexColor(0.10, 0.30, 1.00)
				else
					v.icon:SetVertexColor(1.00, 1.00, 1.00)
				end
				v.outOfRange = nil
			end
		end
	end
end

function SacredBuff:PLAYER_REGEN_DISABLED()
end

function SacredBuff:PLAYER_REGEN_ENABLED()
end

function SacredBuff:ModeChanged()
	local currState = self.db.char.groupMode
	for k in next, self.frame do
		if k == BS["Divine Spirit"] or k == BS["Power Word: Fortitude"] or k == BS["Shadow Protection"] then
			self.frame[k].icon:SetTexture( BS:GetSpellIcon( self.frame[k].states[currState] or self.frame[k].states[1] ) )
		end
	end
end


--[[---------------------------------------------------------------------------------
 OTHER MISCELLANEOUS YET REALLY NEAT-O FUNCTIONS
------------------------------------------------------------------------------------]]
function SacredBuff:UpdateCandleCount()
	self.frame.main:SetText(GetItemCount(17029))
end

function SacredBuff:FindSpell(spell)
	local tbl
	local i = 1
	while true do
		local spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
		if not spellName then
			break
		end
		if spellName == spell then
			if spellRank then
				local r = select(3, spellRank:find(RANK.."%s(%d+)"))
				local r2 = select(3, spellRank:find("Rank%s(%d+)"))
				spellRank = tonumber(r) or 0
				if spellRank == 0 and r2 then
					spellRank = tonumber(r2)
				end
			end
			G:Erase()
			G:SetSpell(i, BOOKTYPE_SPELL)
			local manaCost = select(3, G:Find("(%d+)%s"..MANA, 2, 2, false, true))
			if not manaCost then
				manaCost = "0"
			end
			if not tbl then
				tbl = { i, spellRank, manaCost, spellName }
			else
				if tbl[2] < spellRank then
					tbl[1] = i
					tbl[2] = spellRank
					tbl[3] = manaCost
				end
			end
		end
		i = i + 1
	end
	return tbl
end
